
#if (__VERSION__ > 120)
in vec2 pass_uv;
out vec4 frag_color;
#else
varying vec2 pass_uv;
#endif

uniform sampler2D tex;
uniform sampler2D guided_tex;

uniform float sigma_s;
uniform float sigma_c;
uniform float filter_window;
uniform float shift;

// DCP2 : guided filter of map
// https://www.shadertoy.com/view/XtfSz4

vec3 guidedFilter(sampler2D tex, sampler2D tex_guide, vec2 uv, float sigma_s, float sigma_c)
{
	float wsize = filter_window / (2.0 * shift);
	vec3 res_v = vec3(0.0, 0.0, 0.0);
	float res_w = 0.0;
	vec3 center_g = texture2D(tex_guide, uv).rgb;
	float sigma_i = 0.5*wsize*wsize / sigma_s / sigma_s;
	for (float i = (-1.0 * shift); i <= shift; i++) {
		for (float j = (-1.0 * shift); j <= shift; j++) {
			vec3 tmp_v = texture2D(tex, uv + vec2(float(i)*wsize, float(j)*wsize)).rgb;
			vec3 tmp_g = texture2D(tex_guide, uv + vec2(float(i)*wsize, float(j)*wsize)).rgb;
			vec3 diff_g = (tmp_g - center_g);
			float tmp_w = exp(-(i*i + j * j)*sigma_i);
			tmp_w *= exp(-(dot(diff_g, diff_g) / 2.0 / sigma_c / sigma_c));
			res_v += tmp_v * tmp_w;
			res_w += tmp_w;
		}
	}
	vec3 res = res_v / res_w;
	return res;
}

void main() {
	vec3 c = guidedFilter(tex, guided_tex, pass_uv, sigma_s, sigma_c);
#if (__VERSION__ > 120)
	frag_color = vec4(c, 1.0);
#else
	gl_FragColor = vec4(c, 1.0);
#endif
}