#if (__VERSION__ > 120)
in vec2 pass_uv;
out vec4 frag_color;
#else
varying vec2 pass_uv;
#endif

uniform sampler2D tex;
uniform sampler2D mask_tex;
uniform vec3 lips_color;

// RGB <-> HSV conversion, thanks to http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl
vec3 rgb2hsv(vec3 c)
{
	vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
	vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

	float d = q.x - min(q.w, q.y);
	float e = 1.0e-10;
	return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

// HSV <-> RGB conversion, thanks to http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl
vec3 hsv2rgb(vec3 c)
{
	vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
	vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
	return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
	vec3 color = texture2D(tex, pass_uv).rgb;
	vec3 hsv = rgb2hsv(color);

    vec3 lips_hsv = rgb2hsv(lips_color);
    hsv.x = lips_hsv.x;
    hsv.y = lips_hsv.y;
	//hsv.x = 354.0/360.0;
	//hsv.y = 65.0/100.0;

    vec3 recolor = clamp(hsv2rgb(hsv), 0.0, 1.0);

	float weight = texture2D(mask_tex, pass_uv).x * 0.7;
	color = mix(color, recolor, vec3(weight));

#if (__VERSION__ > 120)
    frag_color = vec4(color, 1.0);
#else
    gl_FragColor = vec4(color, 1.0);
#endif

}