#if (__VERSION__ > 120)
in vec2 pass_uv;
out vec4 frag_color;
#else
varying vec2 pass_uv;
#endif

uniform sampler2D tex;
uniform sampler2D blur_tex;
uniform float weight;

// rgb -> ycbcr (709)
vec3 rgb2ycbcr(vec3 color) {
	return vec3(
		0.2126 * color.r + 0.7152 * color.g + 0.0722 * color.b,
		-0.11457 * color.r - 0.38543 * color.g + 0.5 * color.b,
		0.5 * color.r - 0.45415 * color.g - 0.04585 * color.b
	);
}
vec3 ycbcr2rgb(vec3 ycbcr) {
	return vec3(
		ycbcr.x + 1.5748 * ycbcr.z,
		ycbcr.x - 0.18732 * ycbcr.y - 0.46812 * ycbcr.z,
		ycbcr.x + 1.8556 * ycbcr.y
	);
}
void main() {
	vec3 color = texture2D(tex, pass_uv).rgb;
	vec3 blur_color = texture2D(blur_tex, pass_uv).rgb;

	vec3 ycbcr_color = rgb2ycbcr(color);
	
	float y = ycbcr_color.r + (ycbcr_color.r - blur_color.r) * weight;
	
	color = ycbcr2rgb(vec3(y, ycbcr_color.g, ycbcr_color.b));
	clamp(color, 0.0, 1.0);

#if (__VERSION__ > 120)
	frag_color = vec4(color, 1.0);
#else
	gl_FragColor = vec4(color, 1.0);
#endif
}
