// BLURBOX RGB_FRAGMENT_SHADER_STRING
// https://www.shadertoy.com/view/4slGWn
// MIPMAP 이용해서 BLUR 효과를 주는 방식. https://www.learnopengles.com/tag/mipmap/
// 원래 소스에는 LOD를 사용하나 OpenGL ES 2.0에서 texture2DLod 함수는 Vertext Shader에만 적용가능하므로 BIAS로 대체
// 20200215
// 바둑판과 같이 중간에 생기는 줄을 제거하기 위해 Gaussian Blur Logic 추가: https://github.com/Jam3/glsl-fast-gaussian-blur

precision lowp float;
varying highp vec2 interp_tc;
uniform sampler2D rgb_tex;

//STARTDEF
#define BLURAREA 1.0   // 0.5이면 화면 절반만 적용됨 (디버깅 용도)
#define BLURBIAS 75

#define BIASSTEP 5.0
#define DELTA 10.0

//const vec2 resolution = vec2(640.00, 480.00);
const vec2 resolution = vec2(360.000, 640.000); // window.innerWidth / innerHeight

vec3 blur9(sampler2D image, vec2 uv, vec2 direction, float bias) {
  vec4 color = vec4(0.0);
  vec2 off1 = vec2(1.3846153846) * direction;
  vec2 off2 = vec2(3.2307692308) * direction;
  color += texture2D(image, uv, bias) * 0.2270270270;
  color += texture2D(image, uv + (off1 / resolution), bias) * 0.3162162162;
  color += texture2D(image, uv - (off1 / resolution), bias) * 0.3162162162;
  color += texture2D(image, uv + (off2 / resolution), bias) * 0.0702702703;
  color += texture2D(image, uv - (off2 / resolution), bias) * 0.0702702703;
  return color.xyz;
}
//ENDDEF

void main() {
    //STARTCODE
    //vec2 uv = vec2(gl_FragCoord.xy / iResolution.xy);
    vec2 uv = interp_tc;
    
    //float lod = (5.0 + 5.0 * sin(float(BLURBIAS))) * step(uv.x, BLURAREA);
    float bias = float(BLURBIAS) / 10.0;
    vec3 col = vec3(0.0, 0.0, 0.0);
    
    if (bias < BIASSTEP) {
        // 5.0 이하에 gaussian-blur 적용하면 잔상이 보임
        col = texture2D( rgb_tex, vec2(uv.x, uv.y), bias ).xyz;
    } else {
        // https://github.com/Jam3/glsl-fast-gaussian-blur
        vec2 dir = vec2(0.0, DELTA);
        col += blur9(rgb_tex, uv, dir, bias);
        dir = vec2(DELTA, 0.0);
        col += blur9(rgb_tex, uv, dir, bias);
        col /= 2.0;
    }

    gl_FragColor = vec4( col, 1.0 );
    //ENDCODE
}
