// SNOW_FILMOLD RGB_FRAGMENT_SHADER_STRING
precision mediump float;
varying highp vec2 interp_tc;
uniform sampler2D rgb_tex;

//STARTDEF
#define BEAUTIFY 1
#define OPACITY 0.5
#define BCS 1.0,1.0,1.0

uniform sampler2D texture_filter1;  // LookUpTable
uniform sampler2D texture_filter2;  // grain

// lookuptable
const vec3 W = vec3(0.2125, 0.7154, 0.0721);

vec4 lookup(in vec4 textureColor, in sampler2D lookupTable) {
    #ifndef LUT_NO_CLAMP
        textureColor = clamp(textureColor, 0.0, 1.0);
    #endif

    mediump float blueColor = textureColor.b * 63.0;

    mediump vec2 quad1;
    quad1.y = floor(floor(blueColor) / 8.0);
    quad1.x = floor(blueColor) - (quad1.y * 8.0);

    mediump vec2 quad2;
    quad2.y = floor(ceil(blueColor) / 8.0);
    quad2.x = ceil(blueColor) - (quad2.y * 8.0);

    highp vec2 texPos1;
    texPos1.x = (quad1.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
    texPos1.y = (quad1.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);

    highp vec2 texPos2;
    texPos2.x = (quad2.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
    texPos2.y = (quad2.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);

    lowp vec4 newColor1 = texture2D(lookupTable, texPos1);
    lowp vec4 newColor2 = texture2D(lookupTable, texPos2);

    lowp vec4 newColor = mix(newColor1, newColor2, fract(blueColor));
    return newColor;
}

vec3 BrightnessContrastSaturation(vec3 color, float brt, float con, float sat)
{
    if (brt == 1.0 && con == 1.0 && sat == 1.0) {
        return color;
    }
	vec3 black = vec3(0., 0., 0.);
	vec3 middle = vec3(0.5, 0.5, 0.5);
	float luminance = dot(color, W);
	vec3 gray = vec3(luminance, luminance, luminance);
	
	vec3 brtColor = mix(black, color, brt);
	vec3 conColor = mix(middle, brtColor, con);
	vec3 satColor = mix(gray, conColor, sat);
	return satColor;
}

vec3 ovelayBlender(vec3 Color, vec3 myfilter){
	vec3 filter_result;

	float luminance = dot(myfilter, W);
	
	if(luminance < 0.5)
		filter_result = 2. * myfilter * Color;
	else
		filter_result = 1. - (1. - (2. *(myfilter - 0.5)))*(1. - Color);
		
	return filter_result;
}

// beautify
const vec2 singleStepOffset = vec2(2.0 / (3.000 * 360.000), 2.0 / (3.000 * 640.000));

const vec2 singleStepOffset0 = singleStepOffset * vec2(0.0, -10.0);
const vec2 singleStepOffset1 = singleStepOffset * vec2(0.0, 10.0);
const vec2 singleStepOffset2 = singleStepOffset * vec2(-10.0, 0.0);
const vec2 singleStepOffset3 = singleStepOffset * vec2(10.0, 0.0);
const vec2 singleStepOffset4 = singleStepOffset * vec2(5.0, -8.0);
const vec2 singleStepOffset5 = singleStepOffset * vec2(5.0, 8.0);
const vec2 singleStepOffset6 = singleStepOffset * vec2(-5.0, 8.0);
const vec2 singleStepOffset7 = singleStepOffset * vec2(-5.0, -8.0);
const vec2 singleStepOffset8 = singleStepOffset * vec2(8.0, -5.0);
const vec2 singleStepOffset9 = singleStepOffset * vec2(8.0, 5.0);
const vec2 singleStepOffset10 = singleStepOffset * vec2(-8.0, 5.0);
const vec2 singleStepOffset11 = singleStepOffset * vec2(-8.0, -5.0);
const vec2 singleStepOffset12 = singleStepOffset * vec2(0.0, -6.0);
const vec2 singleStepOffset13 = singleStepOffset * vec2(0.0, 6.0);
const vec2 singleStepOffset14 = singleStepOffset * vec2(6.0, 0.0);
const vec2 singleStepOffset15 = singleStepOffset * vec2(-6.0, 0.0);
const vec2 singleStepOffset16 = singleStepOffset * vec2(-4.0, -4.0);
const vec2 singleStepOffset17 = singleStepOffset * vec2(-4.0, 4.0);
const vec2 singleStepOffset18 = singleStepOffset * vec2(4.0, -4.0);
const vec2 singleStepOffset19 = singleStepOffset * vec2(4.0, 4.0);

const float params = 0.33;
const highp vec3 W2 = vec3(0.299,0.587,0.114);
vec2 blurCoordinates[20];

float hardLight(float color)
{
	if(color <= 0.5)
		color = color * color * 2.0;
	else
		color = 1.0 - ((1.0 - color)*(1.0 - color) * 2.0);
	return color;
}
//ENDDEF

void main() {
	gl_FragColor = texture2D(rgb_tex, interp_tc);

    //STARTCODE
    // beautify  
    int beautify = int(BEAUTIFY);
    if (beautify == 1) {
        vec3 centralColor = gl_FragColor.rgb;

        blurCoordinates[0] = interp_tc.xy + singleStepOffset0;
        blurCoordinates[1] = interp_tc.xy + singleStepOffset1;
        blurCoordinates[2] = interp_tc.xy + singleStepOffset2;
        blurCoordinates[3] = interp_tc.xy + singleStepOffset3;
        blurCoordinates[4] = interp_tc.xy + singleStepOffset4;
        blurCoordinates[5] = interp_tc.xy + singleStepOffset5;
        blurCoordinates[6] = interp_tc.xy + singleStepOffset6;
        blurCoordinates[7] = interp_tc.xy + singleStepOffset7;
        blurCoordinates[8] = interp_tc.xy + singleStepOffset8;
        blurCoordinates[9] = interp_tc.xy + singleStepOffset9;
        blurCoordinates[10] = interp_tc.xy + singleStepOffset10;
        blurCoordinates[11] = interp_tc.xy + singleStepOffset11;
        blurCoordinates[12] = interp_tc.xy + singleStepOffset12;
        blurCoordinates[13] = interp_tc.xy + singleStepOffset13;
        blurCoordinates[14] = interp_tc.xy + singleStepOffset14;
        blurCoordinates[15] = interp_tc.xy + singleStepOffset15;
        blurCoordinates[16] = interp_tc.xy + singleStepOffset16;
        blurCoordinates[17] = interp_tc.xy + singleStepOffset17;
        blurCoordinates[18] = interp_tc.xy + singleStepOffset18;
        blurCoordinates[19] = interp_tc.xy + singleStepOffset19;

        float sampleColor = centralColor.g * 20.0;
        sampleColor += texture2D(rgb_tex, blurCoordinates[0]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[1]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[2]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[3]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[4]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[5]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[6]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[7]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[8]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[9]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[10]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[11]).g;
        sampleColor += texture2D(rgb_tex, blurCoordinates[12]).g * 2.0;
        sampleColor += texture2D(rgb_tex, blurCoordinates[13]).g * 2.0;
        sampleColor += texture2D(rgb_tex, blurCoordinates[14]).g * 2.0;
        sampleColor += texture2D(rgb_tex, blurCoordinates[15]).g * 2.0;
        sampleColor += texture2D(rgb_tex, blurCoordinates[16]).g * 2.0;
        sampleColor += texture2D(rgb_tex, blurCoordinates[17]).g * 2.0;
        sampleColor += texture2D(rgb_tex, blurCoordinates[18]).g * 2.0;
        sampleColor += texture2D(rgb_tex, blurCoordinates[19]).g * 2.0;

        sampleColor = sampleColor / 48.0;

        float highPass = centralColor.g - sampleColor + 0.5;

        for(int i = 0; i < 5;i++)
        {
            highPass = hardLight(highPass);
        }
        float luminance = dot(centralColor, W2);
        float alpha = pow(luminance, params);

        vec3 smoothColor = centralColor + (centralColor-vec3(highPass))*alpha*0.1;
        gl_FragColor = vec4(mix(smoothColor.rgb, max(smoothColor, centralColor), alpha), 1.0);
    }

    // snow filmold
    lowp vec4 textureColor = gl_FragColor;

    // lookuptable
    vec4 newColor = lookup(textureColor, texture_filter1);
    vec4 myfilter = texture2D(texture_filter2, interp_tc);
	vec3 bcs_result = BrightnessContrastSaturation(newColor.rgb, BCS);

    float opacity = float(OPACITY);
    if (opacity == 0.0) {
        opacity = 0.00000001;    // small number (to prevent errors from uniform optimization)
    }
    vec3 after_filter = mix(bcs_result.rgb, ovelayBlender(bcs_result.rgb, myfilter.rgb), opacity);
    gl_FragColor = vec4(after_filter, textureColor.w);
    //ENDCODE
}
