// TOON RGB_FRAGMENT_SHADER_STRING
precision highp float;
varying highp vec2 interp_tc;
uniform sampler2D rgb_tex;

//STARTDEF
varying vec2 leftTextureCoordinate;
varying vec2 rightTextureCoordinate;

varying vec2 topTextureCoordinate;
varying vec2 topLeftTextureCoordinate;
varying vec2 topRightTextureCoordinate;

varying vec2 bottomTextureCoordinate;
varying vec2 bottomLeftTextureCoordinate;
varying vec2 bottomRightTextureCoordinate;

//const float intensity;
const float threshold = 0.5; // 0.2;
const float quantizationLevels = 10.0;

const vec3 W = vec3(0.2125, 0.7154, 0.0721);
//ENDDEF

void main() {
    gl_FragColor = texture2D(rgb_tex, interp_tc);

    //STARTCODE
    vec2 textureCoordinate = interp_tc;
    vec4 textureColor = texture2D(rgb_tex, textureCoordinate);
    
    float bottomLeftIntensity = texture2D(rgb_tex, bottomLeftTextureCoordinate).r;
    float topRightIntensity = texture2D(rgb_tex, topRightTextureCoordinate).r;
    float topLeftIntensity = texture2D(rgb_tex, topLeftTextureCoordinate).r;
    float bottomRightIntensity = texture2D(rgb_tex, bottomRightTextureCoordinate).r;
    float leftIntensity = texture2D(rgb_tex, leftTextureCoordinate).r;
    float rightIntensity = texture2D(rgb_tex, rightTextureCoordinate).r;
    float bottomIntensity = texture2D(rgb_tex, bottomTextureCoordinate).r;
    float topIntensity = texture2D(rgb_tex, topTextureCoordinate).r;
    float h = -topLeftIntensity - 2.0 * topIntensity - topRightIntensity + bottomLeftIntensity + 2.0 * bottomIntensity + bottomRightIntensity;
    float v = -bottomLeftIntensity - 2.0 * leftIntensity - topLeftIntensity + bottomRightIntensity + 2.0 * rightIntensity + topRightIntensity;
    
    float mag = length(vec2(h, v));
    
    vec3 posterizedImageColor = floor((textureColor.rgb * quantizationLevels) + 0.5) / quantizationLevels;
    
    float thresholdTest = 1.0 - step(threshold, mag);
    
    gl_FragColor = vec4(posterizedImageColor * thresholdTest, textureColor.a);
    //ENDCODE
}

//-----------------------------------------------------------------------------
// TOON VERTEX_SHADER_STRING
precision highp float;
varying highp vec2 interp_tc;
attribute vec4 in_pos;
attribute vec4 in_tc;
uniform mat4 texMatrix;

//START_VERTEXDEF
const float texelWidth = 1.0 / 360.000;
const float texelHeight = 1.0 / 640.000;

varying vec2 leftTextureCoordinate;
varying vec2 rightTextureCoordinate;

varying vec2 topTextureCoordinate;
varying vec2 topLeftTextureCoordinate;
varying vec2 topRightTextureCoordinate;

varying vec2 bottomTextureCoordinate;
varying vec2 bottomLeftTextureCoordinate;
varying vec2 bottomRightTextureCoordinate;
//END_VERTEXDEF

void main() {
    gl_Position = in_pos;
    interp_tc = (texMatrix * in_tc).xy;

    //START_VERTEXCODE	
    //gl_TexCoord[0] = gl_MultiTexCoord0;
    //gl_Position = ftransform();
    
    vec2 widthStep = vec2(texelWidth, 0.0);
    vec2 heightStep = vec2(0.0, texelHeight);
    vec2 widthHeightStep = vec2(texelWidth, texelHeight);
    vec2 widthNegativeHeightStep = vec2(texelWidth, -texelHeight);
    
    //vec2 textureCoordinate = vec2(gl_MultiTexCoord0);
    vec2 textureCoordinate = interp_tc;
    leftTextureCoordinate = textureCoordinate - widthStep;
    rightTextureCoordinate = textureCoordinate + widthStep;
    
    topTextureCoordinate = textureCoordinate - heightStep;
    topLeftTextureCoordinate = textureCoordinate - widthHeightStep;
    topRightTextureCoordinate = textureCoordinate + widthNegativeHeightStep;
    
    bottomTextureCoordinate = textureCoordinate + heightStep;
    bottomLeftTextureCoordinate = textureCoordinate - widthNegativeHeightStep;
    bottomRightTextureCoordinate = textureCoordinate + widthHeightStep;
	//END_VERTEXCODE
}
