// CRAYON YUV_FRAGMENT_SHADER_STRING
precision mediump float;
varying highp vec2 interp_tc;
uniform sampler2D y_tex;
uniform sampler2D u_tex;
uniform sampler2D v_tex;

//STARTDEF
#ifdef WEBGL_THREEJS_DETAIL
uniform vec2 stepoffset;
uniform float strength;
#else
//const vec2 stepoffset = vec2(1.0/360.0, 1.0/640.0);
const vec2 stepoffset = vec2(1.0/460.0, 1.0/512.0);
const float strength = 0.5;
#endif

const highp vec3 W = vec3(0.299,0.587,0.114);

const mat3 rgb2yiqMatrix = mat3(
		0.299, 0.587, 0.114,
		0.596,-0.275,-0.321,
		0.212,-0.523, 0.311);

const mat3 yiq2rgbMatrix = mat3(
		1.0, 0.956, 0.621,
		1.0,-0.272,-1.703,
		1.0,-1.106, 0.0);
//ENDDEF

vec4 texture2DYUV(vec2 offset) {
    float y = texture2D(y_tex, interp_tc + offset).r;
    float u = texture2D(u_tex, interp_tc + offset).r - 0.5;
    float v = texture2D(v_tex, interp_tc + offset).r - 0.5;
    return vec4(y + 1.403 * v, y - 0.344 * u - 0.714 * v, y + 1.77 * u, 1);
}

void main() {
    float y = texture2D(y_tex, interp_tc).r;
    float u = texture2D(u_tex, interp_tc).r - 0.5;
    float v = texture2D(v_tex, interp_tc).r - 0.5;
    gl_FragColor = vec4(y + 1.403 * v, y - 0.344 * u - 0.714 * v, y + 1.77 * u, 1);

    //STARTCODE
	//vec4 oralColor = texture2D(inputImageTexture, textureCoordinate);
	vec4 oralColor = gl_FragColor;

	vec3 maxValue = vec3(0.,0.,0.);
	
	for(int i = -2; i<=2; i++)
	{
		for(int j = -2; j<=2; j++)
		{
			//vec4 tempColor = texture2D(inputImageTexture, textureCoordinate+singleStepOffset*vec2(i,j));
#ifdef WEBGL_THREEJS
            vec4 tempColor = texture2D(tDiffuse, vUv + stepoffset*vec2(i,j));
#else
            vec4 tempColor = texture2DYUV(stepoffset*vec2(i,j));
#endif
			maxValue.r = max(maxValue.r,tempColor.r);
			maxValue.g = max(maxValue.g,tempColor.g);
			maxValue.b = max(maxValue.b,tempColor.b);
		}
	}
	
	vec3 textureColor = oralColor.rgb / maxValue;
	
	float gray = dot(textureColor, W);
	float k = 0.223529;
	float alpha = min(gray,k)/k;
	
	textureColor = textureColor * alpha + (1.-alpha)*oralColor.rgb;
	
	vec3 yiqColor = textureColor * rgb2yiqMatrix;

	yiqColor.r = max(0.0,min(1.0,pow(gray,strength)));

	textureColor = yiqColor * yiq2rgbMatrix;
	
	gl_FragColor = vec4(textureColor, oralColor.w);
    //ENDCODE    
}
